package org.derek.melon_proxy;

import com.velocitypowered.api.command.CommandSource;
import com.velocitypowered.api.command.SimpleCommand;
import com.velocitypowered.api.event.Subscribe;
import com.velocitypowered.api.event.proxy.ProxyInitializeEvent;
import com.velocitypowered.api.plugin.Plugin;
import com.velocitypowered.api.proxy.ProxyServer;
import com.velocitypowered.api.proxy.server.RegisteredServer;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.minimessage.MiniMessage;

import javax.inject.Inject;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

@Plugin(id = "melon_proxy", name = "Melon Proxy", version = "1.3", authors = {"Hyakutake@starmc"})
public class MelonProxy {

    private final ProxyServer server;

    @Inject
    public MelonProxy(ProxyServer server) {
        this.server = server;
    }

    @Subscribe
    public void onProxyInitialization(ProxyInitializeEvent event) {
        server.getCommandManager().register("melonproxy", new MelonProxyCommand(server));
    }

    public static class MelonProxyCommand implements SimpleCommand {

        private final ProxyServer server;
        private final MiniMessage miniMessage = MiniMessage.miniMessage();

        public MelonProxyCommand(ProxyServer server) {
            this.server = server;
        }

        @Override
        public void execute(Invocation invocation) {
            CommandSource source = invocation.source();
            String[] args = invocation.arguments();

            if (args.length == 0) {
                source.sendMessage(Component.text("Usage: /melonproxy <broadcast|storebroadcast> <args>"));
                return;
            }

            String subCommand = args[0].toLowerCase();

            switch (subCommand) {
                case "broadcast":
                    handleBroadcast(source, String.join(" ", Arrays.copyOfRange(args, 1, args.length)));
                    break;
                case "storebroadcast":
                    handleStoreBroadcast(source, String.join(" ", Arrays.copyOfRange(args, 1, args.length)));
                    break;
                default:
                    source.sendMessage(Component.text("Usage: /melonproxy <broadcast|storebroadcast> <args>"));
                    break;
            }
        }

        private void handleBroadcast(CommandSource source, String message) {
            if (message.isEmpty()) {
                source.sendMessage(miniMessage.deserialize("<red>Usage: /melonproxy broadcast <message>"));
                return;
            }

            String formattedMessage = convertLegacyColorsToMiniMessage(message);
            Component broadcastMessage = miniMessage.deserialize(
                    "<b><gradient:#E2106E:#FC00FF>Melon Nexu</gradient>" +
                            "<gradient:#FC00FF:#E700FF>s</gradient>" +
                            "<gradient:#E700FF:#B008F8> | Pro</gradient>" +
                            "<gradient:#B008F8:#8000EC>xy »</gradient></b> " +
                            "<white>" + formattedMessage + "</white>"
            );

            for (RegisteredServer registeredServer : server.getAllServers()) {
                for (var player : registeredServer.getPlayersConnected()) {
                    player.sendMessage(broadcastMessage);
                }
            }
        }

        private void handleStoreBroadcast(CommandSource source, String message) {
            if (message.isEmpty()) {
                source.sendMessage(miniMessage.deserialize("<red>Usage: /melonproxy storebroadcast <message>"));
                return;
            }

            String formattedMessage = convertLegacyColorsToMiniMessage(message);
            Component storeBroadcastMessage = miniMessage.deserialize(
                    "<b><gradient:#E2106E:#FC00FF>Melon Nexu</gradient>" +
                            "<gradient:#FC00FF:#E700FF>s</gradient>" +
                            "<gradient:#E700FF:#B008F8> | Sto</gradient>" +
                            "<gradient:#B008F8:#8000EC>re »</gradient></b> " +
                            "<white>" + formattedMessage + "</white>"
            );

            for (RegisteredServer registeredServer : server.getAllServers()) {
                if (!registeredServer.getServerInfo().getName().equalsIgnoreCase("prisons")) {
                    for (var player : registeredServer.getPlayersConnected()) {
                        player.sendMessage(storeBroadcastMessage);
                    }
                }
            }
        }


        private String convertLegacyColorsToMiniMessage(String message) {
            Map<String, String> legacyToMiniMessage = new HashMap<>();
            legacyToMiniMessage.put("&0", "<black>"); legacyToMiniMessage.put("&1", "<dark_blue>"); legacyToMiniMessage.put("&2", "<dark_green>");
            legacyToMiniMessage.put("&4", "<dark_red>"); legacyToMiniMessage.put("&5", "<dark_purple>"); legacyToMiniMessage.put("&3", "<dark_aqua>");
            legacyToMiniMessage.put("&6", "<gold>"); legacyToMiniMessage.put("&7", "<gray>"); legacyToMiniMessage.put("&8", "<dark_gray>");
            legacyToMiniMessage.put("&9", "<blue>"); legacyToMiniMessage.put("&a", "<green>"); legacyToMiniMessage.put("&b", "<aqua>");
            legacyToMiniMessage.put("&c", "<red>"); legacyToMiniMessage.put("&d", "<light_purple>"); legacyToMiniMessage.put("&e", "<yellow>"); 
            legacyToMiniMessage.put("&f", "<white>"); legacyToMiniMessage.put("&k", "<obfuscated>"); legacyToMiniMessage.put("&l", "<bold>");
            legacyToMiniMessage.put("&m", "<strikethrough>"); legacyToMiniMessage.put("&n", "<underline>"); legacyToMiniMessage.put("&o", "<italic>");
            legacyToMiniMessage.put("&r", "<reset>");

            for (Map.Entry<String, String> entry : legacyToMiniMessage.entrySet()) {
                message = message.replace(entry.getKey(), entry.getValue());
            }

            return message;
        }

        @Override
        public boolean hasPermission(Invocation invocation) {
            return invocation.source().hasPermission("melonproxy.broadcast");
        }
    }
}
